#############################################################################
# Copyright (c) 2014-2017 Redfern Electronics Limited. All rights reserved. #
#############################################################################

from pyparsing import *

binary = []
labels = {}

def parse_label(s, l, t):
    labels[t[1]] = len(binary)
    return []

def label_address(name):
    if name in labels:
        return labels[name]
    else:
        return 0

num = Combine(Word('+-' + nums, nums))
ident = Word(alphas, alphas + nums + '_')

label = (Literal(':') + ident).setParseAction(parse_label)

push = (Literal('PUSH') + num).setParseAction(lambda s, l, t: [0x0000 + (int(t[1]) & 0xFF)])
pushf = (Literal('PUSHF') + num).setParseAction(lambda s, l, t: [0x0100 + (int(t[1]) & 0xFF)])
pushl = (Literal('PUSHL') + num).setParseAction(lambda s, l, t: [0x0200 + (int(t[1]) & 0xFF), (int(t[1]) & 0xFF00) >> 8])
pop = (Literal('POP') + num).setParseAction(lambda s, l, t: [0x0300 + (int(t[1]) & 0xFF)])
popf = (Literal('POPF') + num).setParseAction(lambda s, l, t: [0x0400 + (int(t[1]) & 0xFF)])
poprm = Literal('POPRM').setParseAction(lambda s, l, t: [0x0500])
dup = Literal('DUP').setParseAction(lambda s, l, t: [0x0600])

add = Literal('ADD').setParseAction(lambda s, l, t: [0x0800])
sub = Literal('SUB').setParseAction(lambda s, l, t: [0x0900])
mul = Literal('MUL').setParseAction(lambda s, l, t: [0x0A00])
div = Literal('DIV').setParseAction(lambda s, l, t: [0x0B00])
rnd = Literal('RND').setParseAction(lambda s, l, t: [0x0C00])

gte = Literal('GTE').setParseAction(lambda s, l, t: [0x1000])
gt = Literal('GT').setParseAction(lambda s, l, t: [0x1100])
lte = Literal('LTE').setParseAction(lambda s, l, t: [0x1200])
lt = Literal('LT').setParseAction(lambda s, l, t: [0x1300])
eq = Literal('EQ').setParseAction(lambda s, l, t: [0x1400])

bra = (Literal('BRA') + ident).setParseAction(lambda s, l, t: [0x1800 + (label_address(t[1]) & 0xFF), (label_address(t[1]) & 0xFF00) >> 8])
bez = (Literal('BEZ') + ident).setParseAction(lambda s, l, t: [0x1900 + (label_address(t[1]) & 0xFF), (label_address(t[1]) & 0xFF00) >> 8])
bnz = (Literal('BNZ') + ident).setParseAction(lambda s, l, t: [0x1A00 + (label_address(t[1]) & 0xFF), (label_address(t[1]) & 0xFF00) >> 8])

and_op = Literal('AND').setParseAction(lambda s, l, t: [0x2000])
eor_op = Literal('EOR').setParseAction(lambda s, l, t: [0x2100])
not_op = Literal('NOT').setParseAction(lambda s, l, t: [0x2200])
or_op = Literal('OR').setParseAction(lambda s, l, t: [0x2300])

drd = Literal('DRD').setParseAction(lambda s, l, t: [0x2800])
dwr = Literal('DWR').setParseAction(lambda s, l, t: [0x2900])
ard = Literal('ARD').setParseAction(lambda s, l, t: [0x2A00])

mot1 = Literal('MOT1').setParseAction(lambda s, l, t: [0x3000])
mot2 = Literal('MOT2').setParseAction(lambda s, l, t: [0x3100])
ssprk = Literal('SSPRK').setParseAction(lambda s, l, t: [0x3200])
osprk = Literal('OSPRK').setParseAction(lambda s, l, t: [0x3300])
wait = Literal('WAIT').setParseAction(lambda s, l, t: [0x3400])
srv = Literal('SRV').setParseAction(lambda s, l, t: [0x3600])
usons = Literal('USONS').setParseAction(lambda s, l, t: [0x3700])
usonl = Literal('USONL').setParseAction(lambda s, l, t: [0x3800])

stop = Literal('STOP').setParseAction(lambda s, l, t: [0x3FFF])

asm = ZeroOrMore(label |
                 push | pushf | pushl | pop | popf | poprm | dup |
                 add | sub | mul | div | rnd |
                 gte | gt | lte | lt | eq |
                 bra | bez | bnz |
                 and_op | eor_op | not_op | or_op |
                 drd | dwr | ard |
                 mot1 | mot2 | ssprk | osprk | wait | srv | usons | usonl |
                 stop)

def assemble(assembly):
    global labels
    global binary

    binary = []
    labels = {}

    for line in assembly:
        binary.extend(asm.parseString(line, parseAll=True))

    binary = []

    for line in assembly:
        binary.extend(asm.parseString(line))

    return binary
