# -*- coding: utf-8 -*-

#############################################################################
# Copyright (c) 2014-2017 Redfern Electronics Limited. All rights reserved. #
#############################################################################

import collections
import os
import sys
import canvas
import statements
import blocks
import toolbox
import crumble_device
import compile
import vm
import wx
try:
    from pubsub import pub
except ImportError:
    if wx.VERSION[0] < 3:
        from wx.lib.pubsub import setupkwargs
    from wx.lib.pubsub import pub
import xml.etree.cElementTree as ET
import utils
import struct
import urllib2
import lang
from lang import _

version = '1.0.0'
vm_version = 0x15 # Set to zero to force VM update
# vm_version = 0


class CrumbleFrame(wx.Frame):
    def __init__(self):
        super(CrumbleFrame, self).__init__(None, wx.ID_ANY, '', size=(800,600))

        self.filename = ''

        if os.path.exists(utils.get_path() + '/crumble.exe'):
            favicon = wx.Icon('crumble.exe', wx.BITMAP_TYPE_ICO, 16, 16)
            self.SetIcon(favicon)

        self.SetBackgroundColour((0xff,0xff,0xff))

        menu_bar = wx.MenuBar()

        file_menu = wx.Menu()
        file_menu.Append(wx.ID_NEW, '&New\tCtrl+N')
        file_menu.Append(wx.ID_OPEN, '&Open\tCtrl+O')
        file_menu.Append(wx.ID_SAVE, '&Save\tCtrl+S')
        file_menu.Append(wx.ID_SAVEAS, 'Save &As')
        file_menu.AppendSeparator()

        self.Bind(wx.EVT_MENU, self.on_new, id=wx.ID_NEW)
        self.Bind(wx.EVT_MENU, self.on_open, id=wx.ID_OPEN)
        self.Bind(wx.EVT_MENU, self.on_save, id=wx.ID_SAVE)
        self.Bind(wx.EVT_MENU, self.on_save_as, id=wx.ID_SAVEAS)

        exit_menu_item = wx.MenuItem(file_menu, wx.ID_EXIT, '&Exit')
        file_menu.AppendItem(exit_menu_item)
        self.Bind(wx.EVT_MENU, self.on_exit, exit_menu_item)
        self.Bind(wx.EVT_CLOSE, self.on_exit)

        program_menu = wx.Menu()
        send_program_menu_item = program_menu.Append(wx.ID_ANY, 'Program Crumble and &Run')
        stop_menu_item = program_menu.Append(wx.ID_ANY, 'Stop')
        self.Bind(wx.EVT_MENU, self.on_send_program, send_program_menu_item)
        self.Bind(wx.EVT_MENU, self.on_stop, stop_menu_item)

        menu_bar.Append(file_menu, '&File')
        menu_bar.Append(program_menu, '&Program')
        self.SetMenuBar(menu_bar)

        toolbar = self.CreateToolBar()
        run_tool = toolbar.AddLabelTool(wx.ID_ANY, 'Run', wx.Bitmap(utils.get_path() + '/PNG/Run tool.png'))
        stop_tool = toolbar.AddLabelTool(wx.ID_ANY, 'Run', wx.Bitmap(utils.get_path() + '/PNG/Stop tool.png'))
        self.status = wx.StaticText(toolbar, wx.ID_ANY, _('Ready'), size=(320,-1))
        if lang.LANGAUGE == 'chinese':
            self.status.SetFont(wx.Font(16, wx.DEFAULT, wx.NORMAL, wx.NORMAL, faceName='Microsoft YaHei'))
        toolbar.AddControl(self.status)
        toolbar.Realize()

        self.Bind(wx.EVT_TOOL, self.on_send_program, run_tool)
        self.Bind(wx.EVT_TOOL, self.on_stop, stop_tool)

        self.canvas = canvas.Canvas(self, wx.ID_ANY)

        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(self.canvas, 1, flag=wx.EXPAND)
        self.SetSizer(sizer)

        self.canvas.add(toolbox.ToolboxBlock(0, 0), self.canvas.toolboxes)

        self.monitor_timer = wx.Timer(self, wx.ID_ANY)
        self.Bind(wx.EVT_TIMER, self.on_monitor_timer, self.monitor_timer)

        self.check_for_updates()

        self.Show()

    @property
    def filename(self):
        return self._filename

    @filename.setter
    def filename(self, new_name):
        self._filename = new_name
        if self._filename:
            self.Title = 'Crumble Version %s - %s' % (version, os.path.basename(self._filename))
        else:
            self.Title = 'Crumble Version %s' % version

    def check_for_updates(self):
        try:
            file = urllib2.urlopen('http://www.redfernelectronics.co.uk/crumble_version.txt')
            online_version = file.read()
        except:     # Catch all errors :( Firewalls and proxies seem to cause lots of different errors
            pass
        else:
            try:
                (online_major, online_minor, online_patch) = [int(x) for x in online_version.split('.')]
            except ValueError:
                pass
            else:
                (major, minor, patch) = [int(x) for x in version.split('.')]
                update_available = False
                if online_major > major:
                    update_available = True
                elif online_major == major and online_minor > minor:
                    update_available = True
                elif online_major == major and online_minor == minor and online_patch > patch:
                    update_available = True
                if update_available:
                    self.status.Label = 'New software available (%s)' % online_version

    def on_exit(self, event):
        if self.canvas.modified:
            result = wx.MessageBox("Do you want to save changes?", "Crumble",
                             wx.ICON_QUESTION | wx.YES_NO | wx.CANCEL, self)
            if result == wx.YES:
                if not self.save():
                    return
            elif result == wx.CANCEL:
                return

        self.Destroy()

    def on_send_program(self, event):
        start_block = None
        for child in self.canvas.children:
            if type(child) == statements.WhenRunBlock:
                start_block = child

        basic_code = ''
        inits = collections.OrderedDict()
        setups = collections.OrderedDict()

        if start_block:
            blocks = self.canvas.find_all_attached(start_block)
            for block in blocks[1:]:
                basic_code += block.basic_code(inits, setups)

        init_code = ''
        for init in inits.keys():
            init_code += init

        setup_code = ''
        for setup in setups.keys():
            setup_code += setup

        basic_code = init_code + '\n' + basic_code

        machine_code = compile.compile(basic_code)

        # for i in range(0, len(machine_code), 8):
        #     remaining = len(machine_code) - i
        #     if remaining >= 8:
        #         print '0x%04x: ' % i,
        #         print '0x%04x 0x%04x 0x%04x 0x%04x 0x%04x 0x%04x 0x%04x 0x%04x' % tuple(machine_code[i:i+8])
        #     else:
        #         print '0x%04x: ' % i,
        #         for j in range(i, i + remaining):
        #             print '0x%04x ' % machine_code[j],

        try:
            loaded_vm_version = crumble_device.read_vm_version()
            crumble_device.erase_vm_first_page()
            crumble_device.write_byte_code(machine_code)
            if vm_version==0 or loaded_vm_version!=vm_version:
                crumble_device.program_hex(vm.hex_data)
            else:
                crumble_device.program_hex(vm.hex_data, first_page_only=True)
            crumble_device.init_vm()
            crumble_device.send_vm_command(crumble_device.VM_START)
        except IOError:
            self.status.Label = _('ERROR: Crumble not found!')
        else:
            self.status.Label = _('Programming successful')
            self.monitor_start()

    def on_stop(self, event):
        try:
            crumble_device.init_vm()
            # crumble_device.stop_vm()
        except IOError:
            self.status.Label = _('ERROR: Crumble not found!')
        else:
            self.status.Label = _('Stopped')

    def on_new(self, event):
        if self.canvas.modified:
            result = wx.MessageBox(_("Do you want to save changes?"), "Crumble",
                             wx.ICON_QUESTION | wx.YES_NO | wx.CANCEL, self)
            if result == wx.YES:
                if not self.save():
                    return
            elif result == wx.CANCEL:
                return

        self.canvas.toolboxes[0].variables_button.reset()
        self.canvas.toolboxes[0].basic_button.load_tools()

        self.canvas.delete_all()
        self.filename = ''
        self.canvas.modified = False

    def on_save(self, event):
        self.save()

    def on_save_as(self,event):
        self.save(save_as=True)

    def on_open(self, event):
        if self.canvas.modified:
            result = wx.MessageBox(_("Do you want to save changes?"), "Crumble",
                             wx.ICON_QUESTION | wx.YES_NO | wx.CANCEL, self)
            if result == wx.YES:
                if not self.save():
                    return
            elif result == wx.CANCEL:
                return

        open_dialog = wx.FileDialog(self, "Open Crumble file", "", "",
                                    "Crumble files (*.crm)|*.crm", wx.FD_OPEN | wx.FD_FILE_MUST_EXIST)

        if open_dialog.ShowModal() == wx.ID_CANCEL:
            return

        self.filename = open_dialog.GetPath()
        self.canvas.delete_all()
        self.canvas.modified = False
        try:
            tree = ET.ElementTree(file=open_dialog.GetPath())
            root = tree.getroot()
            file_version = root.attrib['version']
            vars_element = root.find('Variables')
            if vars_element:
                vars = []
                for var_element in vars_element:
                    vars.append(var_element.text)
                self.canvas.toolboxes[0].variables_button.set_vars(vars)
            else:
                self.canvas.toolboxes[0].variables_button.reset()
            self.canvas.toolboxes[0].basic_button.load_tools()
            self.canvas_from_xml(root.find('Canvas'))
        except ET.ParseError:
            wx.MessageBox("Could not open: file appears to be corrupted", "Crumble", wx.ICON_ERROR | wx.OK, self)
        except ValueError:
            wx.MessageBox("Error opening file. This file was saved in version %s" % file_version, "Crumble", wx.ICON_ERROR | wx.OK, self)

    def open(self, filename):
        if self.canvas.modified:
            result = wx.MessageBox(_("Do you want to save changes?"), "Crumble",
                             wx.ICON_QUESTION | wx.YES_NO | wx.CANCEL, self)
            if result == wx.YES:
                if not self.save():
                    return
            elif result == wx.CANCEL:
                return

        self.filename = filename
        self.canvas.delete_all()
        self.canvas.modified = False
        try:
            tree = ET.ElementTree(file=filename)
            root = tree.getroot()
            file_version = root.attrib['version']
            vars_element = root.find('Variables')
            if vars_element:
                vars = []
                for var_element in vars_element:
                    vars.append(var_element.text)
                self.canvas.toolboxes[0].variables_button.set_vars(vars)
            else:
                self.canvas.toolboxes[0].variables_button.reset()
            self.canvas.toolboxes[0].basic_button.load_tools()
            self.canvas_from_xml(root.find('Canvas'))
        except ET.ParseError:
            wx.MessageBox("Could not open: file appears to be corrupted :-(", "Crumble", wx.ICON_ERROR | wx.OK, self)
        except ValueError:
            wx.MessageBox("Could not open: file saved in version %s" % file_version, "Crumble", wx.ICON_ERROR | wx.OK, self)

    def save(self, save_as=False):
        if not self.filename or save_as:
            save_dialog = wx.FileDialog(self, "Save Crumble file", "", "",
                                        "Crumble files (*.crm)|*.crm", wx.FD_SAVE | wx.FD_OVERWRITE_PROMPT)
            if save_dialog.ShowModal() == wx.ID_CANCEL:
                return False
            else:
                self.filename = save_dialog.GetPath()
                if self.filename[-4:] != '.crm':
                    self.filename += '.crm'
        canvas_element = self.canvas_to_xml()
        root = ET.Element('CrumbleDocument', {'version': version})
        vars = self.canvas.toolboxes[0].variables_button.get_vars()
        vars_element = ET.Element('Variables')
        for var in vars:
            var_element = ET.Element('Variable')
            var_element.text = var
            vars_element.append(var_element)
        root.append(vars_element)
        root.append(canvas_element)
        tree = ET.ElementTree(root)
        tree.write(self.filename)
        self.canvas.modified = False
        return True

    def canvas_to_xml(self):
        root = ET.Element('Canvas')
        if self.canvas.children:
            for child in self.canvas.children:
                if not child.attached_to and hasattr(child, 'to_xml'):
                    root.append(child.to_xml(position=True))
        return root

    def canvas_from_xml(self, canvas_element):
        for element in canvas_element:
            blocks.from_xml(element, self.canvas)

    def monitor_start(self):
        self.monitor_timer.Start(250)

    def monitor_stop(self):
        pub.sendMessage('crumble.monitor.stopped')
        self.monitor_timer.Stop()

    def on_monitor_timer(self, event):
        try:
            data = crumble_device.read_ram(102, 64)[2:] # 104 is address of stack space, but can't read first byte
        except IOError:
            self.monitor_stop()
            # self.status.Label = 'Crumble disconnected'
        else:
            i = 0
            values = {}
            for var_name in compile.global_vars:
                value = struct.unpack('<h', bytearray(data[i:i+2]))[0]
                values[var_name.replace('_', ' ')] = value
                i+=2
                if i >= len(data):
                    break
            pub.sendMessage('crumble.variable.values', values=values)


class CrumbleApp(wx.App):
    def __init__(self, *args, **kwargs):
        wx.App.__init__(self, *args, **kwargs)

        # This catches events when the app is asked to activate by some other
        # process
        self.Bind(wx.EVT_ACTIVATE_APP, self.OnActivate)

    def OnInit(self):
        self.frame = CrumbleFrame()
        self.frame.Show()

        if wx.Platform != '__WXMAC__':
            if len(sys.argv) > 1:
                self.frame.open(sys.argv[1])

        return True

    def BringWindowToFront(self):
        try: # it's possible for this event to come when the frame is closed
            self.GetTopWindow().Raise()
        except:
            pass

    def OnActivate(self, event):
        # if this is an activate event, rather than something else, like iconize.
        if event.GetActive():
            self.BringWindowToFront()
        event.Skip()

    def MacOpenFile(self, filename):
        pass

    def MacOpenFiles(self, filenames):
        """Called for files droped on dock icon, or opened via finders context menu"""
        self.frame.open(filenames[0])

    def MacReopenApp(self):
        """Called when the doc icon is clicked, and ???"""
        self.BringWindowToFront()

    def MacNewFile(self):
        pass

    def MacPrintFile(self, file_path):
        pass


if __name__=='__main__':
    if len(sys.argv) > 1 and sys.argv[1]=='-v':
        print version,
        exit()

    wx.SetDefaultPyEncoding("utf-8")

    app = CrumbleApp(False)#True)

    app.MainLoop()
