#############################################################################
# Copyright (c) 2014-2017 Redfern Electronics Limited. All rights reserved. #
#############################################################################

import ctypes
from ctypes.util import find_library
import atexit

path = find_library('hidapi-libusb')
if not path:
    path = find_library('hidapi')
if not path:
    raise ImportError('Cannot find hidapi library')

hidapi = ctypes.cdll.LoadLibrary(path)

hidapi.hid_open.argtypes = [ctypes.c_ushort, ctypes.c_ushort, ctypes.c_wchar_p]
hidapi.hid_open.restype = ctypes.c_void_p
hidapi.hid_write.argtypes = [ctypes.c_void_p, ctypes.c_char_p, ctypes.c_size_t]
hidapi.hid_read_timeout.argtypes = [ctypes.c_void_p, ctypes.c_char_p, ctypes.c_size_t, ctypes.c_int]

hidapi.hid_init()

atexit.register(hidapi.hid_exit)

class Device(object):
    def __init__(self):
        self.__handle = None

    def open(self, vid, pid):
        self.__handle = hidapi.hid_open(vid, pid, None)
        if self.__handle is None:
            raise IOError('Error opening device')

    def write(self, data):
        if self.__handle is None:
            raise IOError('Device not open')
        array = ''.join(map(chr, data))
        result = hidapi.hid_write(self.__handle, array, len(array))
        if result == -1:   
            raise IOError('Error writing to device')

    def read(self, length):
        if self.__handle is None:
            raise IOError('Device not open')
        data = ctypes.create_string_buffer(length)
        result = hidapi.hid_read_timeout(self.__handle, data, length, 100)
        if result == -1:
            raise IOError('Error reading from device')
        elif result == 0:
            raise IOError('Read timed out')
        return list(bytearray(data))

    def close(self):
        if self.__handle is not None:
            hidapi.hid_close(self.__handle)
            self.__handle = None

device = Device
