# -*- coding: utf-8 -*-

#############################################################################
# Copyright (c) 2014-2017 Redfern Electronics Limited. All rights reserved. #
#############################################################################

import canvas
import statements
import hardware
import blocks
import wx
from lang import _

class ToolboxButton(blocks.Button):
    def __init__(self, x, y, label, colour):
        super(ToolboxButton, self).__init__(x, y, label, colour)
        self.toolbox_click = True
        self.tools = []

    def load_tools(self):
        toolbox = self.parent
        toolbox.change_tools(self.tools)
        toolbox.highlight_button(self)

    def on_click(self):
        self.load_tools()



class BasicButton(ToolboxButton):
    def __init__(self, x, y):
        super(BasicButton, self).__init__(x, y, _('Basic'), blocks.event_colour)
        self.tools.append(statements.WhenRunBlock(0, 0))
        self.tools.append(statements.DigitalOutputBlock(0, 0))
        self.tools.append(statements.MotorBlock(0, 0))
        self.tools.append(statements.DigitalInputBlock(0, 0))
        self.tools.append(statements.SetSprakleBlock(0, 0))
        self.tools.append(statements.TurnSprakleOffBlock(0, 0))
        self.tools.append(statements.SetAllSpraklesBlock(0, 0))
        self.tools.append(statements.WaitBlock(0, 0))
        self.tools.append(statements.WaitUntilBlock(0, 0))
        self.tools.append(statements.DoUntilBlock(0, 0))
        self.tools.append(statements.DoForeverBlock(0, 0))
        self.tools.append(statements.IfBlock(0, 0))


class IOButton(ToolboxButton):
    def __init__(self, x, y):
        super(IOButton, self).__init__(x, y, _('Input/Output'), blocks.io_colour)
        self.tools.append(statements.DigitalOutputBlock(0, 0))
        self.tools.append(statements.MotorBlock(0, 0))
        self.tools.append(statements.DigitalInputBlock(0, 0))
        self.tools.append(statements.AnalogueInput(0, 0))
        self.tools.append(statements.ServoBlock(0, 0))
        self.tools.append(statements.DistanceBlock(0, 0))


class ControlButton(ToolboxButton):
    def __init__(self, x, y):
        super(ControlButton, self).__init__(x, y, _('Control'), blocks.control_colour)
        self.tools.append(statements.WaitBlock(0, 0))
        self.tools.append(statements.WaitMSBlock(0,0))
        self.tools.append(statements.WaitUntilBlock(0, 0))
        self.tools.append(statements.IfBlock(0, 0))
        self.tools.append(statements.IfElseBlock(0, 0))
        self.tools.append(statements.DoUntilBlock(0, 0))
        self.tools.append(statements.DoForeverBlock(0, 0))
        self.tools.append(statements.DoTimesBlock(0, 0))


class VariablesButton(ToolboxButton):
    def __init__(self, x, y):
        super(VariablesButton, self).__init__(x, y, _('Variables'), blocks.data_colour)
        self.tools.append(statements.AssignmentBlock(0, 0))
        self.tools.append(statements.IncreaseBlock(0, 0))
        self.tools.append(statements.DecreaseBlock(0, 0))
        self.tools.append(NewVariableButton(0, 0, self))

        self.reset()

    def reset(self):
        del self.tools[4:]

        names = ['t', 'u', 'v', 'w', 'x', 'y', 'z']

        for name in names:
            self.add_var(name)

    def get_vars(self):
        vars = []
        for tool in self.tools:
            if isinstance(tool, statements.VariableBlock):
                vars.append(tool.var_name)
        return vars

    def set_vars(self, vars):
        del self.tools[4:]
        for var in vars:
            self.add_var(var)

    def add_var(self, name):
        var = statements.VariableBlock(0, 0)
        var.layout_hint = 'horizontal'
        var.var_name = name
        monitor = statements.VariableMonitorBlock(0, 0)
        monitor.var_name = name
        rename = VariableRenameButton(0, 0, var)
        rename.layout_hint = 'horizontal'
        delete = VariableDeleteButton(0, 0, self, var, monitor, rename)
        delete.layout_hint = 'horizontal'
        self.tools.append(var)
        self.tools.append(delete)
        self.tools.append(rename)
        self.tools.append(monitor)
        return var


class NewVariableButton(blocks.Button):
    def __init__(self, x, y, variables_button):
        super(NewVariableButton, self).__init__(x, y, _('Add New Variable'), blocks.data_colour)
        self.toolbox_click = True
        self.variables_button = variables_button

    def on_click(self):
        toolbox = self.parent
        var = self.variables_button.add_var('NewVar')
        self.variables_button.load_tools()
        var.rename()


class SparklesButton(ToolboxButton):
    def __init__(self, x, y):
        super(SparklesButton, self).__init__(x, y, _('Sparkles'), blocks.sparkle_colour)
        self.tools.append(statements.SetSprakleBlock(0, 0))
        self.tools.append(statements.TurnSprakleOffBlock(0, 0))
        self.tools.append(canvas.Space(0, 0, 5, 5))
        self.tools.append(statements.SetAllSpraklesBlock(0, 0))
        self.tools.append(statements.TurnAllSpraklesOffBlock(0, 0))
        self.tools.append(canvas.Space(0, 0, 5, 5))
        self.tools.append(statements.SetSprakleRGBBlock(0, 0))
        self.tools.append(statements.SetAllSparklesRGBBlock(0, 0))


class OperatorsButton(ToolboxButton):
    def __init__(self, x, y):
        super(OperatorsButton, self).__init__(x, y, _('Operators'), blocks.operator_colour)
        self.tools.append(statements.AddBlock(0, 0))
        self.tools[-1].layout_hint = 'horizontal'
        self.tools.append(statements.SubBlock(0, 0))
        self.tools.append(statements.MultiplyBlock(0, 0))
        self.tools[-1].layout_hint = 'horizontal'
        self.tools.append(statements.DivideBlock(0, 0))

        self.tools.append(canvas.Space(0, 0, 10, 10))

        self.tools.append(statements.EqualityBlock(0, 0))
        self.tools[-1].layout_hint = 'horizontal'
        self.tools.append(statements.NotEqualBlock(0, 0))
        self.tools.append(statements.LessThanBlock(0, 0))
        self.tools[-1].layout_hint = 'horizontal'
        self.tools.append(statements.GreaterThanBlock(0, 0))

        self.tools.append(canvas.Space(0, 0, 10, 10))

        self.tools.append(statements.AndBlock(0, 0))
        self.tools.append(statements.OrBlock(0, 0))
        self.tools.append(statements.NotBlock(0, 0))

        self.tools.append(canvas.Space(0, 0, 10, 10))

        self.tools.append(statements.RandomBlock(0, 0))


class VariableDeleteButton(blocks.Button):
    def __init__(self, x, y, variables_button, variable, monitor, rename):
        super(VariableDeleteButton, self).__init__(x, y, _('del'), (0xE0,0xE0,0xE0,0xFF))
        self.outline.fill = (0x7F,0x7F,0x7F,0xFF)
        self.outline.stroke = (0x7F,0x7F,0x7F,0xFF)
        self.render()
        self.variables_button = variables_button
        self.variable = variable
        self.monitor = monitor
        self.rename = rename
        self.toolbox_click = True

    def on_click(self):
        self.variables_button.tools.remove(self)
        self.variables_button.tools.remove(self.variable)
        self.variables_button.tools.remove(self.monitor)
        self.variables_button.tools.remove(self.rename)
        self.variables_button.load_tools()


class VariableRenameButton(blocks.Button):
    def __init__(self, x, y, variable):
        super(VariableRenameButton, self).__init__(x, y, _('rename'), (0xE0,0xE0,0xE0,0xFF))
        self.outline.fill = (0x7F,0x7F,0x7F,0xFF)
        self.outline.stroke = (0x7F,0x7F,0x7F,0xFF)
        self.render()
        self.variable = variable
        self.toolbox_click = True

    def on_click(self):
        self.variable.rename()


class ToolboxBlock(canvas.Block):
    def __init__(self, x, y):
        super(ToolboxBlock, self).__init__(x, y)
        self.draggable = True
        self.outline = canvas.Rectangle(5, 5, 200, 400, 8, fill=(0xE0,0xE0,0xE0,0xFF), stroke=(0x7F,0x7F,0x7F,0xFF))
        self.add(self.outline)

        self.buttons = []

        self.basic_button = BasicButton(0, 0)
        self.basic_button.layout_hint = 'horizontal'
        self.buttons.append(self.basic_button)

        self.io_button = IOButton(0, 0)
        self.io_button.layout_hint = 'horizontal'
        self.buttons.append(self.io_button)

        self.sparkles_button = SparklesButton(0, 0)
        self.buttons.append(self.sparkles_button)

        self.control_button = ControlButton(0, 0)
        self.control_button.layout_hint = 'horizontal'
        self.buttons.append(self.control_button)

        self.variables_button = VariablesButton(0, 0)
        self.variables_button.layout_hint = 'horizontal'
        self.buttons.append(self.variables_button)

        self.operators_button = OperatorsButton(0, 0)
        self.buttons.append(self.operators_button)

        for button in self.buttons:
            self.add(button)

        self.divider = canvas.Rectangle(0, 0, 2, 1, 0, stroke=(0x7F,0x7F,0x7F,0xFF))
        self.add(self.divider)

        self.bottom_space = canvas.Space(0, 0, 5, 5)
        self.add(self.bottom_space)

        self.buttons[0].on_click()

    def change_tools(self, tools):
        removals = self.children[self.children.index(self.divider) + 1:]
        for child in removals:
            self.remove(child)

        for tool in tools:
            self.add(tool)

        self.add(self.bottom_space)
        self.render()

    def highlight_button(self, b):
        for button in self.buttons:
            if button == b:
                button.highlighted = True
            else:
                button.highlighted = False

    def layout(self):
        top = 10
        left = 10
        x = left
        y = top
        gap = 10
        width = 0
        self.divider.width = 0
        for child in self.children[1:-1]:
            child.x = x
            child.y = y
            box = child.box
            if x + box.width > width:
                width = x + box.width
            if child.layout_hint == 'horizontal':
                x += box.width + gap
            elif box:
                x = left
                y += box.height + gap

        self.outline.height = y - top
        self.outline.width = width
        self.bottom_space.y = self.outline.height + self.outline.y
        self.divider.width = width - left
